@@ -4,7 +4,7 @@
 
 import datetime
 import re
-from typing import List, Optional, Union
+from typing import ClassVar, Iterator, List, Optional, Union
 
 import dateutil.parser
 import dateutil.relativedelta as rdelta
@@ -13,6 +13,68 @@ import typepy
 from .__version__ import __author__, __copyright__, __email__, __license__, __version__
 
 
+def _to_norm_relativedelta(td: Union[datetime.timedelta, rdelta.relativedelta]) -> rdelta.relativedelta:
+    if isinstance(td, rdelta.relativedelta):
+        return td.normalized()
+
+    return rdelta.relativedelta(seconds=int(td.total_seconds()), microseconds=td.microseconds).normalized()
+
+
+def _compare_relativedelta(lhs: rdelta.relativedelta, rhs: rdelta.relativedelta) -> int:
+    if lhs.years < rhs.years:
+        return -1
+    if lhs.years > rhs.years:
+        return 1
+
+    if lhs.months < rhs.months:
+        return -1
+    if lhs.months > rhs.months:
+        return 1
+
+    if lhs.days < rhs.days:
+        return -1
+    if lhs.days > rhs.days:
+        return 1
+
+    if lhs.hours < rhs.hours:
+        return -1
+    if lhs.hours > rhs.hours:
+        return 1
+
+    if lhs.minutes < rhs.minutes:
+        return -1
+    if lhs.minutes > rhs.minutes:
+        return 1
+
+    if lhs.seconds < rhs.seconds:
+        return -1
+    if lhs.seconds > rhs.seconds:
+        return 1
+
+    if lhs.microseconds < rhs.microseconds:
+        return -1
+    if lhs.microseconds > rhs.microseconds:
+        return 1
+
+    return 0
+
+
+def _compare_timedelta(lhs: Union[datetime.timedelta, rdelta.relativedelta], seconds: int) -> int:
+    return _compare_relativedelta(_to_norm_relativedelta(lhs), rdelta.relativedelta(seconds=seconds))
+
+
+def _normalize_datetime_value(
+    value: Union[datetime.datetime, str, None], timezone: Optional[str]
+) -> Optional[datetime.datetime]:
+    if value is None:
+        return None
+
+    try:
+        return typepy.type.DateTime(value, strict_level=typepy.StrictLevel.MIN, timezone=timezone).convert()
+    except typepy.TypeConversionError as e:
+        raise ValueError(e)
+
+
 class DateTimeRange:
     """
     A class that represents a range of datetime.
@@ -49,15 +111,15 @@ class DateTimeRange:
         .. seealso:: :py:meth:`.get_end_time_str`
     """
 
-    NOT_A_TIME_STR = "NaT"
+    NOT_A_TIME_STR: ClassVar[str] = "NaT"
 
     def __init__(
         self,
-        start_datetime=None,
-        end_datetime=None,
-        start_time_format="%Y-%m-%dT%H:%M:%S%z",
-        end_time_format="%Y-%m-%dT%H:%M:%S%z",
-    ):
+        start_datetime: Union[datetime.datetime, str, None] = None,
+        end_datetime: Union[datetime.datetime, str, None] = None,
+        start_time_format: str = "%Y-%m-%dT%H:%M:%S%z",
+        end_time_format: str = "%Y-%m-%dT%H:%M:%S%z",
+    ) -> None:
         self.set_time_range(start_datetime, end_datetime)
 
         self.start_time_format = start_time_format
@@ -66,49 +128,79 @@ class DateTimeRange:
         self.is_output_elapse = False
         self.separator = " - "
 
-    def __repr__(self):
-        if self.is_output_elapse:
+    def __repr__(self) -> str:
+        if self.is_output_elapse and self.end_datetime and self.start_datetime:
             suffix = f" ({self.end_datetime - self.start_datetime})"
         else:
             suffix = ""
 
         return self.separator.join((self.get_start_time_str(), self.get_end_time_str())) + suffix
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if not isinstance(other, DateTimeRange):
             return False
 
-        return all(
-            [self.start_datetime == other.start_datetime, self.end_datetime == other.end_datetime]
-        )
+        return all([self.start_datetime == other.start_datetime, self.end_datetime == other.end_datetime])
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         if not isinstance(other, DateTimeRange):
             return True
 
-        return any(
-            [self.start_datetime != other.start_datetime, self.end_datetime != other.end_datetime]
-        )
+        return any([self.start_datetime != other.start_datetime, self.end_datetime != other.end_datetime])
 
-    def __add__(self, other):
-        return DateTimeRange(self.start_datetime + other, self.end_datetime + other)
+    def __add__(self, other: Union[datetime.timedelta, rdelta.relativedelta]) -> "DateTimeRange":
+        if self.start_datetime is None and self.end_datetime is None:
+            raise TypeError("range is not set")
 
-    def __iadd__(self, other):
-        self.set_start_datetime(self.start_datetime + other)
-        self.set_end_datetime(self.end_datetime + other)
+        start_datetime = self.start_datetime
+        if start_datetime:
+            start_datetime += other
+
+        end_datetime = self.end_datetime
+        if end_datetime:
+            end_datetime += other
+
+        return DateTimeRange(start_datetime, end_datetime)
+
+    def __iadd__(self, other: Union[datetime.timedelta, rdelta.relativedelta]) -> "DateTimeRange":
+        if self.start_datetime is None and self.end_datetime is None:
+            raise TypeError("range is not set")
+
+        if self.start_datetime:
+            self.set_start_datetime(self.start_datetime + other)
+
+        if self.end_datetime:
+            self.set_end_datetime(self.end_datetime + other)
 
         return self
 
-    def __sub__(self, other):
-        return DateTimeRange(self.start_datetime - other, self.end_datetime - other)
+    def __sub__(self, other: Union[datetime.timedelta, rdelta.relativedelta]) -> "DateTimeRange":
+        if self.start_datetime is None and self.end_datetime is None:
+            raise TypeError("range is not set")
 
-    def __isub__(self, other):
-        self.set_start_datetime(self.start_datetime - other)
-        self.set_end_datetime(self.end_datetime - other)
+        start_datetime = self.start_datetime
+        if start_datetime:
+            start_datetime -= other
+
+        end_datetime = self.end_datetime
+        if end_datetime:
+            end_datetime -= other
+
+        return DateTimeRange(start_datetime, end_datetime)
+
+    def __isub__(self, other: Union[datetime.timedelta, rdelta.relativedelta]) -> "DateTimeRange":
+        if self.start_datetime is None and self.end_datetime is None:
+            raise TypeError("range is not set")
+
+        if self.start_datetime:
+            self.set_start_datetime(self.start_datetime - other)
+
+        if self.end_datetime:
+            self.set_end_datetime(self.end_datetime - other)
 
         return self
 
-    def __contains__(self, x):
+    def __contains__(self, x: Union[datetime.datetime, "DateTimeRange", str]) -> bool:
         """
         :param x:
             |datetime|/``DateTimeRange`` instance to compare.
@@ -140,22 +232,25 @@ class DateTimeRange:
         """
 
         self.validate_time_inversion()
+        assert self.start_datetime
+        assert self.end_datetime
 
         if isinstance(x, DateTimeRange):
+            x.validate_time_inversion()
+            assert x.start_datetime
+            assert x.end_datetime
+
             return x.start_datetime >= self.start_datetime and x.end_datetime <= self.end_datetime
 
-        try:
-            value = dateutil.parser.parse(x)
-        except (TypeError, AttributeError):
-            value = x
+        value = dateutil.parser.parse(x) if isinstance(x, str) else x
 
         return self.start_datetime <= value <= self.end_datetime
 
     @property
-    def start_datetime(self):
+    def start_datetime(self) -> Optional[datetime.datetime]:
         """
         :return: Start time of the time range.
-        :rtype: datetime.datetime
+        :rtype: Optional[datetime.datetime]
 
         :Sample Code:
             .. code:: python
@@ -172,10 +267,10 @@ class DateTimeRange:
         return self.__start_datetime
 
     @property
-    def end_datetime(self):
+    def end_datetime(self) -> Optional[datetime.datetime]:
         """
         :return: End time of the time range.
-        :rtype: datetime.datetime
+        :rtype: Optional[datetime.datetime]
 
         :Sample Code:
             .. code:: python
@@ -192,7 +287,7 @@ class DateTimeRange:
         return self.__end_datetime
 
     @property
-    def timedelta(self):
+    def timedelta(self) -> datetime.timedelta:
         """
         :return:
             (|attr_end_datetime| - |attr_start_datetime|) as |timedelta|
@@ -210,9 +305,14 @@ class DateTimeRange:
                 datetime.timedelta(0, 600)
         """
 
+        if self.start_datetime is None:
+            raise TypeError("Must set start_datetime")
+        if self.end_datetime is None:
+            raise TypeError("Must set end_datetime")
+
         return self.end_datetime - self.start_datetime
 
-    def is_set(self):
+    def is_set(self) -> bool:
         """
         :return:
             |True| if both |attr_start_datetime| and
@@ -238,7 +338,7 @@ class DateTimeRange:
 
         return all([self.start_datetime is not None, self.end_datetime is not None])
 
-    def validate_time_inversion(self):
+    def validate_time_inversion(self) -> None:
         """
         Check time inversion of the time range.
 
@@ -268,14 +368,15 @@ class DateTimeRange:
             # for python2/3 compatibility
             raise TypeError
 
+        assert self.start_datetime
+        assert self.end_datetime
+
         if self.start_datetime > self.end_datetime:
             raise ValueError(
-                "time inversion found: {:s} > {:s}".format(
-                    str(self.start_datetime), str(self.end_datetime)
-                )
+                "time inversion found: {:s} > {:s}".format(str(self.start_datetime), str(self.end_datetime))
             )
 
-    def is_valid_timerange(self):
+    def is_valid_timerange(self) -> bool:
         """
         :return:
             |True| if the time range is
@@ -311,9 +412,17 @@ class DateTimeRange:
 
         return self.is_set()
 
-    def is_intersection(self, x):
+    def is_intersection(
+        self,
+        x: "DateTimeRange",
+        intersection_threshold: Union[datetime.timedelta, rdelta.relativedelta, None] = None,
+    ) -> bool:
         """
         :param DateTimeRange x: Value to compare
+        :param Union[datetime.timedelta, dateutil.relativedelta.relativedelta, None] intersection_threshold:
+            Minimum time constraint that an intersection must have.
+            Defaults to ``None`` (no constraint).
+
         :return: |True| if intersect with ``x``
         :rtype: bool
 
@@ -330,9 +439,9 @@ class DateTimeRange:
                 True
         """
 
-        return self.intersection(x).is_set()
+        return self.intersection(x, intersection_threshold).is_set()
 
-    def get_start_time_str(self):
+    def get_start_time_str(self) -> str:
         """
         :return:
             |attr_start_datetime| as |str| formatted with
@@ -355,12 +464,15 @@ class DateTimeRange:
                 2015/03/22 10:00:00
         """
 
+        if self.start_datetime is None:
+            return self.NOT_A_TIME_STR
+
         try:
             return self.start_datetime.strftime(self.start_time_format)
         except AttributeError:
             return self.NOT_A_TIME_STR
 
-    def get_end_time_str(self):
+    def get_end_time_str(self) -> str:
         """
         :return:
             |attr_end_datetime| as a |str| formatted with
@@ -383,12 +495,15 @@ class DateTimeRange:
                 2015/03/22 10:10:00
         """
 
+        if self.end_datetime is None:
+            return self.NOT_A_TIME_STR
+
         try:
             return self.end_datetime.strftime(self.end_time_format)
         except AttributeError:
             return self.NOT_A_TIME_STR
 
-    def get_timedelta_second(self):
+    def get_timedelta_second(self) -> float:
         """
         :return: (|attr_end_datetime| - |attr_start_datetime|) as seconds
         :rtype: float
@@ -407,7 +522,7 @@ class DateTimeRange:
 
         return self.timedelta.total_seconds()
 
-    def set_start_datetime(self, value, timezone=None):
+    def set_start_datetime(self, value: Union[datetime.datetime, str, None], timezone: Optional[str] = None) -> None:
         """
         Set the start time of the time range.
 
@@ -430,9 +545,9 @@ class DateTimeRange:
                 2015-03-22T10:00:00+0900 - NaT
         """
 
-        self.__start_datetime = self.__normalize_datetime_value(value, timezone)
+        self.__start_datetime = _normalize_datetime_value(value, timezone)
 
-    def set_end_datetime(self, value, timezone=None):
+    def set_end_datetime(self, value: Union[datetime.datetime, str, None], timezone: Optional[str] = None) -> None:
         """
         Set the end time of the time range.
 
@@ -454,9 +569,11 @@ class DateTimeRange:
                 NaT - 2015-03-22T10:10:00+0900
         """
 
-        self.__end_datetime = self.__normalize_datetime_value(value, timezone)
+        self.__end_datetime = _normalize_datetime_value(value, timezone)
 
-    def set_time_range(self, start, end):
+    def set_time_range(
+        self, start: Union[datetime.datetime, str, None], end: Union[datetime.datetime, str, None]
+    ) -> None:
         """
         :param datetime.datetime/str start: |param_start_datetime|
         :param datetime.datetime/str end: |param_end_datetime|
@@ -479,61 +596,7 @@ class DateTimeRange:
         self.set_start_datetime(start)
         self.set_end_datetime(end)
 
-    @staticmethod
-    def __compare_relativedelta(lhs, rhs):
-        if lhs.years < rhs.years:
-            return -1
-        if lhs.years > rhs.years:
-            return 1
-
-        if lhs.months < rhs.months:
-            return -1
-        if lhs.months > rhs.months:
-            return 1
-
-        if lhs.days < rhs.days:
-            return -1
-        if lhs.days > rhs.days:
-            return 1
-
-        if lhs.hours < rhs.hours:
-            return -1
-        if lhs.hours > rhs.hours:
-            return 1
-
-        if lhs.minutes < rhs.minutes:
-            return -1
-        if lhs.minutes > rhs.minutes:
-            return 1
-
-        if lhs.seconds < rhs.seconds:
-            return -1
-        if lhs.seconds > rhs.seconds:
-            return 1
-
-        if lhs.microseconds < rhs.microseconds:
-            return -1
-        if lhs.microseconds > rhs.microseconds:
-            return 1
-
-        return 0
-
-    def __compare_timedelta(self, lhs, seconds):
-        try:
-            rhs = datetime.timedelta(seconds=seconds)
-
-            if lhs < rhs:
-                return -1
-            if lhs > rhs:
-                return 1
-
-            return 0
-        except TypeError:
-            return self.__compare_relativedelta(
-                lhs.normalized(), rdelta.relativedelta(seconds=seconds)
-            )
-
-    def range(self, step):
+    def range(self, step: Union[datetime.timedelta, rdelta.relativedelta]) -> Iterator[datetime.datetime]:
         """
         Return an iterator object.
 
@@ -560,7 +623,8 @@ class DateTimeRange:
                 2015-01-04 00:00:00+09:00
         """
 
-        if self.__compare_timedelta(step, 0) == 0:
+        cmp_step_w_zero = _compare_timedelta(step, seconds=0)
+        if cmp_step_w_zero == 0:
             raise ValueError("step must be not zero")
 
         is_inversion = False
@@ -569,25 +633,40 @@ class DateTimeRange:
         except ValueError:
             is_inversion = True
 
-        if not is_inversion:
-            if self.__compare_timedelta(step, seconds=0) < 0:
-                raise ValueError(f"invalid step: expect greater than 0, actual={step}")
-        else:
-            if self.__compare_timedelta(step, seconds=0) > 0:
-                raise ValueError(f"invalid step: expect less than 0, actual={step}")
+        assert self.start_datetime
+        assert self.end_datetime
 
         current_datetime = self.start_datetime
-        while current_datetime <= self.end_datetime:
-            yield current_datetime
-            current_datetime = current_datetime + step
 
-    def intersection(self, x):
+        if not is_inversion:
+            if cmp_step_w_zero < 0:
+                raise ValueError(f"invalid step: expect greater than 0, actual={step}")
+
+            while current_datetime <= self.end_datetime:
+                yield current_datetime
+                current_datetime = current_datetime + step
+        else:
+            if cmp_step_w_zero > 0:
+                raise ValueError(f"invalid step: expect less than 0, actual={step}")
+
+            while current_datetime >= self.end_datetime:
+                yield current_datetime
+                current_datetime = current_datetime + step
+
+    def intersection(
+        self,
+        x: "DateTimeRange",
+        intersection_threshold: Union[datetime.timedelta, rdelta.relativedelta, None] = None,
+    ) -> "DateTimeRange":
         """
-        Newly set a time range that overlaps
-        the input and the current time range.
+        Create a new time range that overlaps the input and the current time range.
+        If no overlaps found, return a time range that set ``None`` for both start and end time.
 
         :param DateTimeRange x:
             Value to compute intersection with the current time range.
+        :param Union[datetime.timedelta, dateutil.relativedelta.relativedelta, None] intersection_threshold:
+            Minimum time constraint that an intersection must have.
+            Defaults to ``None`` (no constraint).
 
         :Sample Code:
             .. code:: python
@@ -604,6 +683,10 @@ class DateTimeRange:
 
         self.validate_time_inversion()
         x.validate_time_inversion()
+        assert self.start_datetime
+        assert self.end_datetime
+        assert x.start_datetime
+        assert x.end_datetime
 
         if any([x.start_datetime in self, self.start_datetime in x]):
             start_datetime = max(self.start_datetime, x.start_datetime)
@@ -612,6 +695,21 @@ class DateTimeRange:
             start_datetime = None
             end_datetime = None
 
+        if intersection_threshold is not None:
+            assert start_datetime is not None
+            assert end_datetime is not None
+            delta = end_datetime - start_datetime
+
+            if (
+                _compare_relativedelta(
+                    _to_norm_relativedelta(delta),
+                    _to_norm_relativedelta(intersection_threshold),
+                )
+                < 0
+            ):
+                start_datetime = None
+                end_datetime = None
+
         return DateTimeRange(
             start_datetime=start_datetime,
             end_datetime=end_datetime,
@@ -619,7 +717,7 @@ class DateTimeRange:
             end_time_format=self.end_time_format,
         )
 
-    def subtract(self, x):
+    def subtract(self, x: "DateTimeRange") -> List["DateTimeRange"]:
         """
         Remove a time range from this one and return the result.
 
@@ -658,10 +756,7 @@ class DateTimeRange:
             ]
 
         # Case 2, full overlap, subtraction results in empty set
-        if (
-            overlap.start_datetime == self.start_datetime
-            and overlap.end_datetime == self.end_datetime
-        ):
+        if overlap.start_datetime == self.start_datetime and overlap.end_datetime == self.end_datetime:
             return []
 
         # Case 3, overlap on start
@@ -702,10 +797,9 @@ class DateTimeRange:
             ),
         ]
 
-    def encompass(self, x):
+    def encompass(self, x: "DateTimeRange") -> "DateTimeRange":
         """
-        Newly set a time range that encompasses
-        the input and the current time range.
+        Create a new time range that encompasses the input and the current time range.
 
         :param DateTimeRange x:
             Value to compute encompass with the current time range.
@@ -725,6 +819,10 @@ class DateTimeRange:
 
         self.validate_time_inversion()
         x.validate_time_inversion()
+        assert self.start_datetime
+        assert self.end_datetime
+        assert x.start_datetime
+        assert x.end_datetime
 
         return DateTimeRange(
             start_datetime=min(self.start_datetime, x.start_datetime),
@@ -733,7 +831,7 @@ class DateTimeRange:
             end_time_format=self.end_time_format,
         )
 
-    def truncate(self, percentage):
+    def truncate(self, percentage: float) -> None:
         """
         Truncate ``percentage`` / 2 [%] of whole time from first and last time.
 
@@ -766,12 +864,15 @@ class DateTimeRange:
 
         discard_time = self.timedelta // int(100) * int(percentage / 2)
 
-        self.__start_datetime += discard_time
-        self.__end_datetime -= discard_time
+        if self.__start_datetime:
+            self.__start_datetime += discard_time
+
+        if self.__end_datetime:
+            self.__end_datetime -= discard_time
 
     def split(self, separator: Union[str, datetime.datetime]) -> List["DateTimeRange"]:
         """
-        Split the DateTimerange in two DateTimerange at a specifit datetime.
+        Split the DateTimerange in two DateTimerange at a specific datetime.
 
         :param Union[str, datetime.datetime] separator:
             Date and time to split the DateTimeRange.
@@ -792,11 +893,10 @@ class DateTimeRange:
 
         self.validate_time_inversion()
 
-        separatingseparation = self.__normalize_datetime_value(separator, timezone=None)
+        separatingseparation = _normalize_datetime_value(separator, timezone=None)
+        assert separatingseparation
 
-        if (separatingseparation not in self) or (
-            separatingseparation in (self.start_datetime, self.end_datetime)
-        ):
+        if (separatingseparation not in self) or (separatingseparation in (self.start_datetime, self.end_datetime)):
             return [
                 DateTimeRange(
                     start_datetime=self.start_datetime,
@@ -821,22 +921,11 @@ class DateTimeRange:
             ),
         ]
 
-    def __normalize_datetime_value(self, value, timezone):
-        if value is None:
-            return None
-
-        try:
-            return typepy.type.DateTime(
-                value, strict_level=typepy.StrictLevel.MIN, timezone=timezone
-            ).convert()
-        except typepy.TypeConversionError as e:
-            raise ValueError(e)
-
     @classmethod
     def from_range_text(
         cls,
         range_text: str,
-        separator: str = "-",
+        separator: str = r"\s+\-\s+",
         start_time_format: Optional[str] = None,
         end_time_format: Optional[str] = None,
     ) -> "DateTimeRange":
@@ -847,17 +936,17 @@ class DateTimeRange:
             e.g. ``2021-01-23T10:00:00+0400 - 2021-01-232T10:10:00+0400``
 
         :param str separator:
-            Text that separating the ``range_text``.
+            Regular expression that separating the ``range_text`` to start and end time.
 
         :return: DateTimeRange
             Created instance.
         """
 
-        dattime_ranges = re.split(r"\s+{}\s+".format(re.escape(separator)), range_text.strip())
-        if len(dattime_ranges) != 2:
-            raise ValueError("range_text should include two datetime that separated by hyphen")
+        datetime_ranges = re.split(separator, range_text.strip())
+        if len(datetime_ranges) != 2:
+            raise ValueError(f"range_text should include two datetime that separated by hyphen: got={datetime_ranges}")
 
-        start, end = dattime_ranges
+        start, end = datetime_ranges
         kwargs = {
             "start_datetime": start,
             "end_datetime": end,
