@@ -162,9 +162,7 @@ class TestDateTimeRange_repr:
             [None, ISO_TIME_FORMAT, None, ISO_TIME_FORMAT, " - ", False, "NaT - NaT"],
         ],
     )
-    def test_normal(
-        self, start, start_format, end, end_format, separator, is_output_elapse, expected
-    ):
+    def test_normal(self, start, start_format, end, end_format, separator, is_output_elapse, expected):
         dtr = DateTimeRange(start, end, start_format, end_format)
         dtr.separator = separator
         dtr.is_output_elapse = is_output_elapse
@@ -225,9 +223,7 @@ class TestDateTimeRange_repr:
             ],
         ],
     )
-    def test_exception(
-        self, start, start_format, end, end_format, separator, is_output_elapse, expected
-    ):
+    def test_exception(self, start, start_format, end, end_format, separator, is_output_elapse, expected):
         dtr = DateTimeRange(start, end, start_format, end_format)
         dtr.separator = separator
         dtr.is_output_elapse = is_output_elapse
@@ -325,12 +321,18 @@ class TestDateTimeRange_add:
                 timedelta(seconds=-10 * 60),
                 DateTimeRange("2015-03-22T09:50:00", "2015-03-22T10:00:00"),
             ],
+            [
+                DateTimeRange("2015-03-22T10:00:00", "2015-03-22T10:10:00"),
+                relativedelta(hours=+6),
+                DateTimeRange("2015-03-22T16:00:00", "2015-03-22T16:10:00"),
+            ],
         ],
     )
     def test_normal(self, value, add_value, expected):
         new_datetimerange = value + add_value
 
         assert new_datetimerange == expected
+        assert value != new_datetimerange
 
     @pytest.mark.parametrize(
         ["value", "expected"],
@@ -347,15 +349,23 @@ class TestDateTimeRange_add:
 
 class TestDateTimeRange_iadd:
     def test_normal(self):
-        value = DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900")
+        value1 = DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900")
+        value2 = DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900")
         expected = DateTimeRange("2015-03-22T10:10:00+0900", "2015-03-22T10:20:00+0900")
 
-        value += timedelta(seconds=10 * 60)
-        assert value == expected
+        value1 += timedelta(seconds=10 * 60)
+        assert value1 == expected
+
+        value2 += relativedelta(seconds=10 * 60)
+        assert value2 == expected
 
     @pytest.mark.parametrize(
         ["value", "expected"],
-        [["2015-03-22T10:10:00+0900", TypeError], [1, TypeError], [None, TypeError]],
+        [
+            ["2015-03-22T10:10:00+0900", TypeError],
+            [1, TypeError],
+            [None, TypeError],
+        ],
     )
     def test_exception(self, datetimerange_normal, value, expected):
         with pytest.raises(TypeError):
@@ -373,10 +383,19 @@ class TestDateTimeRange_sub:
 
         new_datetimerange = value - timedelta(seconds=10 * 60)
         assert new_datetimerange == expected
+        assert value != new_datetimerange
+
+        new_datetimerange = value - relativedelta(seconds=10 * 60)
+        assert new_datetimerange == expected
+        assert value != new_datetimerange
 
     @pytest.mark.parametrize(
         ["value", "expected"],
-        [["2015-03-22T10:10:00+0900", TypeError], [1, TypeError], [None, TypeError]],
+        [
+            ["2015-03-22T10:10:00+0900", TypeError],
+            [1, TypeError],
+            [None, TypeError],
+        ],
     )
     def test_exception(self, datetimerange_normal, value, expected):
         with pytest.raises(TypeError):
@@ -389,15 +408,23 @@ class TestDateTimeRange_sub:
 
 class TestDateTimeRange_isub:
     def test_normal(self):
-        value = DateTimeRange("2015-03-22T10:10:00+0900", "2015-03-22T10:20:00+0900")
+        value1 = DateTimeRange("2015-03-22T10:10:00+0900", "2015-03-22T10:20:00+0900")
+        value2 = DateTimeRange("2015-03-22T10:10:00+0900", "2015-03-22T10:20:00+0900")
         expected = DateTimeRange("2015-03-22T10:00:00+0900", "2015-03-22T10:10:00+0900")
 
-        value -= timedelta(seconds=10 * 60)
-        assert value == expected
+        value1 -= timedelta(seconds=10 * 60)
+        assert value1 == expected
+
+        value2 -= relativedelta(seconds=10 * 60)
+        assert value2 == expected
 
     @pytest.mark.parametrize(
         ["value", "expected"],
-        [["2015-03-22T10:10:00+0900", TypeError], [1, TypeError], [None, TypeError]],
+        [
+            ["2015-03-22T10:10:00+0900", TypeError],
+            [1, TypeError],
+            [None, TypeError],
+        ],
     )
     def test_exception(self, datetimerange_normal, value, expected):
         with pytest.raises(TypeError):
@@ -604,7 +631,9 @@ class TestDateTimeRange_range:
         ],
     )
     def test_normal(self, value, step, expected):
-        for value_item, expected_item in zip(value.range(step), expected):
+        results = list(value.range(step))
+        assert len(results) == len(expected)
+        for value_item, expected_item in zip(results, expected):
             assert value_item == expected_item
 
     @pytest.mark.parametrize(
@@ -699,6 +728,35 @@ class TestDateTimeRange_is_intersection:
     def test_normal(self, lhs, rhs, expected):
         assert lhs.is_intersection(rhs) == expected
 
+    @pytest.mark.parametrize(
+        ["lhs", "rhs", "threshold", "expected"],
+        [
+            [
+                DateTimeRange("2015-01-22T09:50:00 JST", "2015-01-22T10:00:00 JST"),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-03-22T10:20:00 JST"),
+                timedelta(seconds=0),
+                True,
+            ],
+            [
+                DateTimeRange("2015-01-22T09:50:00 JST", "2015-01-22T10:00:00 JST"),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-03-22T10:20:00 JST"),
+                timedelta(seconds=1),
+                False,
+            ],
+            [
+                DateTimeRange("2015-01-22T09:50:00 JST", "2015-01-22T10:00:01 JST"),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-03-22T10:20:00 JST"),
+                timedelta(seconds=1),
+                True,
+            ],
+        ],
+    )
+    def test_normal_w_intersection_threshold(self, lhs, rhs, threshold, expected):
+        lhs_org = deepcopy(lhs)
+
+        assert lhs.is_intersection(rhs, threshold) == expected
+        assert lhs == lhs_org
+
     @pytest.mark.parametrize(
         ["lhs", "rhs", "expected"],
         [
@@ -906,7 +964,11 @@ class TestDateTimeRange_set_start_datetime:
         assert dtr.start_datetime == expected
 
     @pytest.mark.parametrize(
-        ["value", "expected"], [["invalid time string", ValueError], ["3.3.5", ValueError]]
+        ["value", "expected"],
+        [
+            ["invalid time string", ValueError],
+            ["3.3.5", ValueError],
+        ],
     )
     def test_null_start(self, datetimerange_null_start, value, expected):
         with pytest.raises(expected):
@@ -936,7 +998,11 @@ class TestDateTimeRange_set_end_datetime:
         assert dtr.end_datetime == expected
 
     @pytest.mark.parametrize(
-        ["value", "expected"], [["invalid time string", ValueError], ["3.3.5", ValueError]]
+        ["value", "expected"],
+        [
+            ["invalid time string", ValueError],
+            ["3.3.5", ValueError],
+        ],
     )
     def test_null_start(self, datetimerange_null_start, value, expected):
         with pytest.raises(expected):
@@ -1021,6 +1087,35 @@ class TestDateTimeRange_intersection:
         assert lhs.intersection(rhs) == expected
         assert lhs == lhs_org
 
+    @pytest.mark.parametrize(
+        ["lhs", "rhs", "threshold", "expected"],
+        [
+            [
+                DateTimeRange("2015-01-22T09:50:00 JST", "2015-01-22T10:00:00 JST"),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-03-22T10:20:00 JST"),
+                timedelta(seconds=0),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-01-22T10:00:00 JST"),
+            ],
+            [
+                DateTimeRange("2015-01-22T09:50:00 JST", "2015-01-22T10:00:00 JST"),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-03-22T10:20:00 JST"),
+                timedelta(seconds=1),
+                DateTimeRange(None, None),
+            ],
+            [
+                DateTimeRange("2015-01-22T09:50:00 JST", "2015-01-22T10:00:01 JST"),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-03-22T10:20:00 JST"),
+                timedelta(seconds=1),
+                DateTimeRange("2015-01-22T10:00:00 JST", "2015-01-22T10:00:01 JST"),
+            ],
+        ],
+    )
+    def test_normal_w_intersection_threshold(self, lhs, rhs, threshold, expected):
+        lhs_org = deepcopy(lhs)
+
+        assert lhs.intersection(rhs, threshold) == expected
+        assert lhs == lhs_org
+
     @pytest.mark.parametrize(
         ["lhs", "rhs", "expected"],
         [
@@ -1181,18 +1276,18 @@ class TestDateTimeRange_from_range_text:
         ["value", "separator", "expected"],
         [
             [
-                "{} - {}".format(START_DATETIME_TEXT, END_DATETIME_TEXT),
-                "-",
+                f"{START_DATETIME_TEXT} - {END_DATETIME_TEXT}",
+                r"\s+\-\s+",
                 DateTimeRange(START_DATETIME_TEXT, END_DATETIME_TEXT),
             ],
             [
-                "{} to {}".format(START_DATETIME_TEXT, END_DATETIME_TEXT),
+                f"{START_DATETIME_TEXT} to {END_DATETIME_TEXT}",
                 "to",
                 DateTimeRange(START_DATETIME_TEXT, END_DATETIME_TEXT),
             ],
             [
-                "{}  -  {}".format(START_DATETIME_TEXT, END_DATETIME_TEXT),
-                "-",
+                f"{START_DATETIME_TEXT}  -  {END_DATETIME_TEXT}",
+                r"\s+\-\s+",
                 DateTimeRange(START_DATETIME_TEXT, END_DATETIME_TEXT),
             ],
         ],
@@ -1207,7 +1302,7 @@ class TestDateTimeRange_from_range_text:
         ["value", "time_format", "expected"],
         [
             [
-                "{} - {}".format(START_DATETIME_TEXT, END_DATETIME_TEXT),
+                f"{START_DATETIME_TEXT} - {END_DATETIME_TEXT}",
                 r"%Y-%m-%d",
                 DateTimeRange(
                     START_DATETIME_TEXT,
@@ -1219,9 +1314,7 @@ class TestDateTimeRange_from_range_text:
         ],
     )
     def test_normal_time_format(self, value, time_format, expected):
-        dtr = DateTimeRange.from_range_text(
-            value, start_time_format=time_format, end_time_format=time_format
-        )
+        dtr = DateTimeRange.from_range_text(value, start_time_format=time_format, end_time_format=time_format)
         assert dtr == expected
         assert dtr.start_time_format == time_format
         assert dtr.end_time_format == time_format
